declare const corrupt: (unreachable: never) => never;

type AnyFunction = (...args: any[]) => unknown;
type ExhaustiveUnion<Union extends string | boolean> = Union extends string ? {
    [Key in Union]: (value: Key) => any;
} & ExhaustiveFallback : Union extends boolean ? {
    [Key in `${Union}`]: (value: Key extends 'true' ? true : false) => any;
} & ExhaustiveFallback : never;
type ExhaustiveTag<Union extends object, Tag extends keyof Union, Values extends Union[Tag] = Union[Tag]> = Values extends string ? {
    [Key in Values]: (value: Extract<Union, {
        [K in Tag]: Key;
    }>) => any;
} & ExhaustiveFallback : Values extends boolean ? {
    [Key in `${Values}`]: (value: Extract<Union, {
        [K in Tag]: Key extends 'true' ? true : false;
    }>) => any;
} & ExhaustiveFallback : never;
type ExhaustiveFallback = {
    /**
     * Default fallback
     *
     * @description
     * When declared, "exhaustive" will fallback to this callback
     * instead of throwing an unreachable error
     */
    _?: () => any;
};
/**
 * Ensures no extra values are passed to the object
 */
type ValidateKeys<T, U> = [keyof T] extends [keyof U] ? T : {
    [Key in keyof U]: Key extends keyof T ? T[Key] : never;
};
declare function exhaustive<Union extends string | boolean, Match extends ExhaustiveUnion<Union> = ExhaustiveUnion<Union>, Output = Match[keyof Match] extends AnyFunction ? ReturnType<Match[keyof Match]> : never>(union: Union, match: ValidateKeys<Match, ExhaustiveUnion<Union>>): Output;
declare function exhaustive<Union extends object, Tag extends keyof Union, Match extends ExhaustiveTag<Union, Tag> = ExhaustiveTag<Union, Tag>, Output = Match[keyof Match] extends AnyFunction ? ReturnType<Match[keyof Match]> : never>(union: Union, tag: Tag, match: ValidateKeys<Match, ExhaustiveTag<Union, Tag>>): Output;
declare namespace exhaustive {
    var tag: <Union extends object, Tag extends keyof Union, Match extends ExhaustiveTag<Union, Tag, Union[Tag]> = ExhaustiveTag<Union, Tag, Union[Tag]>, Output = Match[keyof Match] extends AnyFunction ? ReturnType<Match[keyof Match]> : never>(union: Union, tag: Tag, match: ValidateKeys<Match, ExhaustiveTag<Union, Tag, Union[Tag]>>) => Output;
}

export { ExhaustiveTag, ExhaustiveUnion, corrupt, exhaustive };
